@@ -22,12 +22,15 @@ pub use schema::*;
 
 mod nan_val_cnt_visitor;
 pub(crate) use nan_val_cnt_visitor::*;
-
-pub(crate) mod delete_file_manager;
+pub(crate) mod caching_delete_file_loader;
+/// Delete File loader
+pub mod delete_file_loader;
+pub(crate) mod delete_filter;
 
 mod reader;
 pub(crate) mod record_batch_projector;
 pub(crate) mod record_batch_transformer;
 mod value;
+
 pub use reader::*;
 pub use value::*;
