@@ -39,7 +39,7 @@ pub enum ErrorKind {
     /// Iceberg data is invalid.
     ///
     /// This error is returned when we try to read a table from iceberg but
-    /// failed to parse it's metadata or data file correctly.
+    /// failed to parse its metadata or data file correctly.
     ///
     /// The table could be invalid or corrupted.
     DataInvalid,
@@ -60,6 +60,9 @@ pub enum ErrorKind {
     ///
     /// This error is returned when given iceberg feature is not supported.
     FeatureUnsupported,
+
+    /// Catalog commit failed due to outdated metadata
+    CatalogCommitConflicts,
 }
 
 impl ErrorKind {
@@ -80,6 +83,7 @@ impl From<ErrorKind> for &'static str {
             ErrorKind::NamespaceAlreadyExists => "NamespaceAlreadyExists",
             ErrorKind::NamespaceNotFound => "NamespaceNotFound",
             ErrorKind::PreconditionFailed => "PreconditionFailed",
+            ErrorKind::CatalogCommitConflicts => "CatalogCommitConflicts",
         }
     }
 }
@@ -134,6 +138,8 @@ pub struct Error {
 
     source: Option<anyhow::Error>,
     backtrace: Backtrace,
+
+    retryable: bool,
 }
 
 impl Display for Error {
@@ -225,9 +231,17 @@ impl Error {
             // `Backtrace::capture()` will check if backtrace has been enabled
             // internally. It's zero cost if backtrace is disabled.
             backtrace: Backtrace::capture(),
+
+            retryable: false,
         }
     }
 
+    /// Set retryable of the error.
+    pub fn with_retryable(mut self, retryable: bool) -> Self {
+        self.retryable = retryable;
+        self
+    }
+
     /// Add more context in error.
     pub fn with_context(mut self, key: &'static str, value: impl Into<String>) -> Self {
         self.context.push((key, value.into()));
@@ -306,6 +320,11 @@ impl Error {
         self.kind
     }
 
+    /// Return error's retryable status
+    pub fn retryable(&self) -> bool {
+        self.retryable
+    }
+
     /// Return error's message.
     #[inline]
     pub fn message(&self) -> &str {
