@@ -21,12 +21,12 @@ use std::sync::Arc;
 use uuid::Uuid;
 
 use super::{
-    FormatVersion, MetadataLog, PartitionSpec, PartitionSpecBuilder, PartitionStatisticsFile,
-    Schema, SchemaRef, Snapshot, SnapshotLog, SnapshotReference, SnapshotRetention, SortOrder,
-    SortOrderRef, StatisticsFile, StructType, TableMetadata, UnboundPartitionSpec,
-    DEFAULT_PARTITION_SPEC_ID, DEFAULT_SCHEMA_ID, MAIN_BRANCH, ONE_MINUTE_MS,
-    PROPERTY_METADATA_PREVIOUS_VERSIONS_MAX, PROPERTY_METADATA_PREVIOUS_VERSIONS_MAX_DEFAULT,
-    RESERVED_PROPERTIES, UNPARTITIONED_LAST_ASSIGNED_ID,
+    DEFAULT_PARTITION_SPEC_ID, DEFAULT_SCHEMA_ID, FormatVersion, MAIN_BRANCH, MetadataLog,
+    ONE_MINUTE_MS, PROPERTY_METADATA_PREVIOUS_VERSIONS_MAX,
+    PROPERTY_METADATA_PREVIOUS_VERSIONS_MAX_DEFAULT, PartitionSpec, PartitionSpecBuilder,
+    PartitionStatisticsFile, RESERVED_PROPERTIES, Schema, SchemaRef, Snapshot, SnapshotLog,
+    SnapshotReference, SnapshotRetention, SortOrder, SortOrderRef, StatisticsFile, StructType,
+    TableMetadata, UNPARTITIONED_LAST_ASSIGNED_ID, UnboundPartitionSpec,
 };
 use crate::error::{Error, ErrorKind, Result};
 use crate::{TableCreation, TableUpdate};
@@ -54,6 +54,7 @@ pub struct TableMetadataBuilder {
     last_added_order_id: Option<i64>,
     // None if this is a new table (from_metadata) method not used
     previous_history_entry: Option<MetadataLog>,
+    last_updated_ms: Option<i64>,
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -119,7 +120,9 @@ impl TableMetadataBuilder {
                 refs: HashMap::default(),
                 statistics: HashMap::new(),
                 partition_statistics: HashMap::new(),
+                encryption_keys: HashMap::new(),
             },
+            last_updated_ms: None,
             changes: vec![],
             last_added_schema_id: Some(schema_id),
             last_added_spec_id: None,
@@ -136,7 +139,6 @@ impl TableMetadataBuilder {
     }
 
     /// Creates a new table metadata builder from the given metadata to modify it.
-
     /// `current_file_location` is the location where the current version
     /// of the metadata file is stored. This is used to update the metadata log.
     /// If `current_file_location` is `None`, the metadata log will not be updated.
@@ -156,6 +158,7 @@ impl TableMetadataBuilder {
             last_added_schema_id: None,
             last_added_spec_id: None,
             last_added_order_id: None,
+            last_updated_ms: None,
         }
     }
 
@@ -309,7 +312,7 @@ impl TableMetadataBuilder {
         Ok(self)
     }
 
-    /// Set the location of the table metadata, stripping any trailing slashes.
+    /// Set the location of the table, stripping any trailing slashes.
     pub fn set_location(mut self, location: String) -> Self {
         let location = location.trim_end_matches('/').to_string();
         if self.metadata.location != location {
@@ -349,7 +352,7 @@ impl TableMetadataBuilder {
                     "Cannot add snapshot with sequence number {} older than last sequence number {}",
                     snapshot.sequence_number(),
                     self.metadata.last_sequence_number
-                )
+                ),
             ));
         }
 
@@ -368,13 +371,17 @@ impl TableMetadataBuilder {
             }
         }
 
-        if snapshot.timestamp_ms() - self.metadata.last_updated_ms < -ONE_MINUTE_MS {
+        let max_last_updated = self
+            .last_updated_ms
+            .unwrap_or_default()
+            .max(self.metadata.last_updated_ms);
+        if snapshot.timestamp_ms() - max_last_updated < -ONE_MINUTE_MS {
             return Err(Error::new(
                 ErrorKind::DataInvalid,
                 format!(
                     "Invalid snapshot timestamp {}: before last updated timestamp {}",
                     snapshot.timestamp_ms(),
-                    self.metadata.last_updated_ms
+                    max_last_updated
                 ),
             ));
         }
@@ -384,7 +391,7 @@ impl TableMetadataBuilder {
             snapshot: snapshot.clone(),
         });
 
-        self.metadata.last_updated_ms = snapshot.timestamp_ms();
+        self.last_updated_ms = Some(snapshot.timestamp_ms());
         self.metadata.last_sequence_number = snapshot.sequence_number();
         self.metadata
             .snapshots
@@ -483,19 +490,23 @@ impl TableMetadataBuilder {
             matches!(update, TableUpdate::AddSnapshot { snapshot: snap } if snap.snapshot_id() == snapshot.snapshot_id())
         });
         if is_added_snapshot {
-            self.metadata.last_updated_ms = snapshot.timestamp_ms();
+            self.last_updated_ms = Some(snapshot.timestamp_ms());
         }
 
         // Current snapshot id is set only for the main branch
         if ref_name == MAIN_BRANCH {
             self.metadata.current_snapshot_id = Some(snapshot.snapshot_id());
-            if self.metadata.last_updated_ms == i64::default() {
-                self.metadata.last_updated_ms = chrono::Utc::now().timestamp_millis();
+            let timestamp_ms = if let Some(last_updated_ms) = self.last_updated_ms {
+                last_updated_ms
+            } else {
+                let last_updated_ms = chrono::Utc::now().timestamp_millis();
+                self.last_updated_ms = Some(last_updated_ms);
+                last_updated_ms
             };
 
             self.metadata.snapshot_log.push(SnapshotLog {
                 snapshot_id: snapshot.snapshot_id(),
-                timestamp_ms: self.metadata.last_updated_ms,
+                timestamp_ms,
             });
         }
 
@@ -751,17 +762,19 @@ impl TableMetadataBuilder {
             ));
         }
 
-        let schemaless_spec =
-            self.metadata
-                .partition_specs
-                .get(&spec_id)
-                .ok_or_else(|| {
-                    Error::new(
-                ErrorKind::DataInvalid,
-                format!("Cannot set default partition spec to unknown spec with id: '{spec_id}'",),
-            )
-                })?
-                .clone();
+        let schemaless_spec = self
+            .metadata
+            .partition_specs
+            .get(&spec_id)
+            .ok_or_else(|| {
+                Error::new(
+                    ErrorKind::DataInvalid,
+                    format!(
+                        "Cannot set default partition spec to unknown spec with id: '{spec_id}'",
+                    ),
+                )
+            })?
+            .clone();
         let spec = Arc::unwrap_or_clone(schemaless_spec);
         let spec_type = spec.partition_type(self.get_current_schema()?)?;
         self.metadata.default_spec = Arc::new(spec);
@@ -831,7 +844,7 @@ impl TableMetadataBuilder {
         if sort_order_found {
             if self.last_added_order_id != Some(new_order_id) {
                 self.changes.push(TableUpdate::AddSortOrder {
-                    sort_order: sort_order.clone(),
+                    sort_order: sort_order.clone().with_order_id(new_order_id),
                 });
                 self.last_added_order_id = Some(new_order_id);
             }
@@ -911,9 +924,9 @@ impl TableMetadataBuilder {
 
     /// Build the table metadata.
     pub fn build(mut self) -> Result<TableMetadataBuildResult> {
-        if self.metadata.last_updated_ms == i64::default() {
-            self.metadata.last_updated_ms = chrono::Utc::now().timestamp_millis();
-        }
+        self.metadata.last_updated_ms = self
+            .last_updated_ms
+            .unwrap_or_else(|| chrono::Utc::now().timestamp_millis());
 
         // Check compatibility of the current schema to the default partition spec and sort order.
         // We use the `get_xxx` methods from the builder to avoid using the panicking
@@ -1200,6 +1213,37 @@ impl TableMetadataBuilder {
     fn highest_sort_order_id(&self) -> Option<i64> {
         self.metadata.sort_orders.keys().max().copied()
     }
+
+    /// Remove schemas by their ids from the table metadata.
+    /// Does nothing if a schema id is not present. Active schemas should not be removed.
+    pub fn remove_schemas(mut self, schema_id_to_remove: &[i32]) -> Result<Self> {
+        if schema_id_to_remove.contains(&self.metadata.current_schema_id) {
+            return Err(Error::new(
+                ErrorKind::DataInvalid,
+                "Cannot remove current schema",
+            ));
+        }
+
+        if schema_id_to_remove.is_empty() {
+            return Ok(self);
+        }
+
+        let mut removed_schemas = Vec::with_capacity(schema_id_to_remove.len());
+        self.metadata.schemas.retain(|id, _schema| {
+            if schema_id_to_remove.contains(id) {
+                removed_schemas.push(*id);
+                false
+            } else {
+                true
+            }
+        });
+
+        self.changes.push(TableUpdate::RemoveSchemas {
+            schema_ids: removed_schemas,
+        });
+
+        Ok(self)
+    }
 }
 
 impl From<TableMetadataBuildResult> for TableMetadata {
@@ -1210,12 +1254,19 @@ impl From<TableMetadataBuildResult> for TableMetadata {
 
 #[cfg(test)]
 mod tests {
+    use std::fs::File;
+    use std::io::BufReader;
+    use std::thread::sleep;
+
     use super::*;
+    use crate::TableIdent;
+    use crate::io::FileIOBuilder;
     use crate::spec::{
         BlobMetadata, NestedField, NullOrder, Operation, PartitionSpec, PrimitiveType, Schema,
         SnapshotRetention, SortDirection, SortField, StructType, Summary, Transform, Type,
         UnboundPartitionField,
     };
+    use crate::table::Table;
 
     const TEST_LOCATION: &str = "s3://bucket/test/location";
     const LAST_ASSIGNED_COLUMN_ID: i32 = 3;
@@ -1360,12 +1411,10 @@ mod tests {
                 NestedField::required(
                     13,
                     "struct",
-                    Type::Struct(StructType::new(vec![NestedField::required(
-                        14,
-                        "nested",
-                        Type::Primitive(PrimitiveType::Long),
-                    )
-                    .into()])),
+                    Type::Struct(StructType::new(vec![
+                        NestedField::required(14, "nested", Type::Primitive(PrimitiveType::Long))
+                            .into(),
+                    ])),
                 )
                 .into(),
                 NestedField::required(15, "c", Type::Primitive(PrimitiveType::Long)).into(),
@@ -1401,12 +1450,10 @@ mod tests {
                 NestedField::required(
                     3,
                     "struct",
-                    Type::Struct(StructType::new(vec![NestedField::required(
-                        5,
-                        "nested",
-                        Type::Primitive(PrimitiveType::Long),
-                    )
-                    .into()])),
+                    Type::Struct(StructType::new(vec![
+                        NestedField::required(5, "nested", Type::Primitive(PrimitiveType::Long))
+                            .into(),
+                    ])),
                 )
                 .into(),
                 NestedField::required(4, "c", Type::Primitive(PrimitiveType::Long)).into(),
@@ -1918,19 +1965,21 @@ mod tests {
 
         let builder = builder.add_snapshot(snapshot.clone()).unwrap();
 
-        assert!(builder
-            .clone()
-            .set_ref(MAIN_BRANCH, SnapshotReference {
-                snapshot_id: 10,
-                retention: SnapshotRetention::Branch {
-                    min_snapshots_to_keep: Some(10),
-                    max_snapshot_age_ms: None,
-                    max_ref_age_ms: None,
-                },
-            })
-            .unwrap_err()
-            .to_string()
-            .contains("Cannot set 'main' to unknown snapshot: '10'"));
+        assert!(
+            builder
+                .clone()
+                .set_ref(MAIN_BRANCH, SnapshotReference {
+                    snapshot_id: 10,
+                    retention: SnapshotRetention::Branch {
+                        min_snapshots_to_keep: Some(10),
+                        max_snapshot_age_ms: None,
+                        max_ref_age_ms: None,
+                    },
+                })
+                .unwrap_err()
+                .to_string()
+                .contains("Cannot set 'main' to unknown snapshot: '10'")
+        );
 
         let build_result = builder
             .set_ref(MAIN_BRANCH, SnapshotReference {
@@ -2112,9 +2161,10 @@ mod tests {
             .build()
             .unwrap_err();
 
-        assert!(err
-            .to_string()
-            .contains("Cannot find partition source field"));
+        assert!(
+            err.to_string()
+                .contains("Cannot find partition source field")
+        );
     }
 
     #[test]
@@ -2233,9 +2283,10 @@ mod tests {
         let err = builder
             .set_branch_snapshot(snapshot, MAIN_BRANCH)
             .unwrap_err();
-        assert!(err
-            .to_string()
-            .contains("Cannot add snapshot with sequence number"));
+        assert!(
+            err.to_string()
+                .contains("Cannot add snapshot with sequence number")
+        );
     }
 
     #[test]
@@ -2341,4 +2392,108 @@ mod tests {
         assert_eq!(build_result.metadata.partition_statistics.len(), 0);
         assert_eq!(build_result.changes.len(), 0);
     }
+
+    #[test]
+    fn last_update_increased_for_property_only_update() {
+        let builder = builder_without_changes(FormatVersion::V2);
+
+        let metadata = builder.build().unwrap().metadata;
+        let last_updated_ms = metadata.last_updated_ms;
+        sleep(std::time::Duration::from_millis(2));
+
+        let build_result = metadata
+            .into_builder(Some(
+                "s3://bucket/test/location/metadata/metadata1.json".to_string(),
+            ))
+            .set_properties(HashMap::from_iter(vec![(
+                "foo".to_string(),
+                "bar".to_string(),
+            )]))
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert!(
+            build_result.metadata.last_updated_ms > last_updated_ms,
+            "{} > {}",
+            build_result.metadata.last_updated_ms,
+            last_updated_ms
+        );
+    }
+
+    #[test]
+    fn test_construct_default_main_branch() {
+        // Load the table without ref
+        let file = File::open(format!(
+            "{}/testdata/table_metadata/{}",
+            env!("CARGO_MANIFEST_DIR"),
+            "TableMetadataV2Valid.json"
+        ))
+        .unwrap();
+        let reader = BufReader::new(file);
+        let resp = serde_json::from_reader::<_, TableMetadata>(reader).unwrap();
+
+        let table = Table::builder()
+            .metadata(resp)
+            .metadata_location("s3://bucket/test/location/metadata/v1.json".to_string())
+            .identifier(TableIdent::from_strs(["ns1", "test1"]).unwrap())
+            .file_io(FileIOBuilder::new("memory").build().unwrap())
+            .build()
+            .unwrap();
+
+        assert_eq!(
+            table.metadata().refs.get(MAIN_BRANCH).unwrap().snapshot_id,
+            table.metadata().current_snapshot_id().unwrap()
+        );
+    }
+
+    #[test]
+    fn test_active_schema_cannot_be_removed() {
+        let builder = builder_without_changes(FormatVersion::V2);
+        builder.remove_schemas(&[0]).unwrap_err();
+    }
+
+    #[test]
+    fn test_remove_schemas() {
+        let file = File::open(format!(
+            "{}/testdata/table_metadata/{}",
+            env!("CARGO_MANIFEST_DIR"),
+            "TableMetadataV2Valid.json"
+        ))
+        .unwrap();
+        let reader = BufReader::new(file);
+        let resp = serde_json::from_reader::<_, TableMetadata>(reader).unwrap();
+
+        let table = Table::builder()
+            .metadata(resp)
+            .metadata_location("s3://bucket/test/location/metadata/v1.json".to_string())
+            .identifier(TableIdent::from_strs(["ns1", "test1"]).unwrap())
+            .file_io(FileIOBuilder::new("memory").build().unwrap())
+            .build()
+            .unwrap();
+
+        assert_eq!(2, table.metadata().schemas.len());
+
+        {
+            // can not remove active schema
+            let meta_data_builder = table.metadata().clone().into_builder(None);
+            meta_data_builder.remove_schemas(&[1]).unwrap_err();
+        }
+
+        let mut meta_data_builder = table.metadata().clone().into_builder(None);
+        meta_data_builder = meta_data_builder.remove_schemas(&[0]).unwrap();
+        let build_result = meta_data_builder.build().unwrap();
+        assert_eq!(1, build_result.metadata.schemas.len());
+        assert_eq!(1, build_result.metadata.current_schema_id);
+        assert_eq!(1, build_result.metadata.current_schema().schema_id());
+        assert_eq!(1, build_result.changes.len());
+
+        let remove_schema_ids =
+            if let TableUpdate::RemoveSchemas { schema_ids } = &build_result.changes[0] {
+                schema_ids
+            } else {
+                unreachable!("Expected RemoveSchema change")
+            };
+        assert_eq!(remove_schema_ids, &[0]);
+    }
 }
