@@ -23,6 +23,6 @@ pub use error::*;
 
 mod physical_plan;
 mod schema;
-mod table;
+pub mod table;
 pub use table::table_provider_factory::IcebergTableProviderFactory;
 pub use table::*;
