@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+pub mod metadata_table;
 pub mod table_provider_factory;
 
 use std::any::Any;
@@ -28,8 +29,10 @@ use datafusion::error::Result as DFResult;
 use datafusion::logical_expr::{Expr, TableProviderFilterPushDown};
 use datafusion::physical_plan::ExecutionPlan;
 use iceberg::arrow::schema_to_arrow_schema;
+use iceberg::inspect::MetadataTableType;
 use iceberg::table::Table;
 use iceberg::{Catalog, Error, ErrorKind, NamespaceIdent, Result, TableIdent};
+use metadata_table::IcebergMetadataTableProvider;
 
 use crate::physical_plan::scan::IcebergTableScan;
 
@@ -107,6 +110,13 @@ impl IcebergTableProvider {
             schema,
         })
     }
+
+    pub(crate) fn metadata_table(&self, r#type: MetadataTableType) -> IcebergMetadataTableProvider {
+        IcebergMetadataTableProvider {
+            table: self.table.clone(),
+            r#type,
+        }
+    }
 }
 
 #[async_trait]
