@@ -17,12 +17,14 @@
 
 use std::process::Command;
 
+use tracing::info;
+
 pub fn run_command(mut cmd: Command, desc: impl ToString) {
     let desc = desc.to_string();
-    log::info!("Starting to {}, command: {:?}", &desc, cmd);
+    info!("Starting to {}, command: {:?}", &desc, cmd);
     let exit = cmd.status().unwrap();
     if exit.success() {
-        log::info!("{} succeed!", desc)
+        info!("{} succeed!", desc)
     } else {
         panic!("{} failed: {:?}", desc, exit);
     }
@@ -30,12 +32,12 @@ pub fn run_command(mut cmd: Command, desc: impl ToString) {
 
 pub fn get_cmd_output_result(mut cmd: Command, desc: impl ToString) -> Result<String, String> {
     let desc = desc.to_string();
-    log::info!("Starting to {}, command: {:?}", &desc, cmd);
+    info!("Starting to {}, command: {:?}", &desc, cmd);
     let result = cmd.output();
     match result {
         Ok(output) => {
             if output.status.success() {
-                log::info!("{} succeed!", desc);
+                info!("{} succeed!", desc);
                 Ok(String::from_utf8(output.stdout).unwrap())
             } else {
                 Err(format!("{} failed with rc: {:?}", desc, output.status))
